/*
 * Copyright 1999 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * The contents of this file are subject to the Sun(TM) Community Source
 * License, Jini(TM) Technology Core Platform, v. 1.0 (the "License");
 * you may not use this file except in compliance with the License. You
 * may obtain a copy of the License at http://www.sun.com/jini. Software
 * distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
 * License for the specific language governing rights and limitations
 * under the License.
 * 
 * The Original Code is Jini Technology Core Platform code, version 1.0.
 * The Developer of the Original Code is Sun Microsystems, Inc. Portions
 * created by Sun Microsystems, Inc. are Copyright (C) 1999 Sun
 * Microsystems, Inc. All Rights Reserved.
 * 
 * Contributor(s): Sun Microsystems, Inc.
 * 
 * The contents of this file comply with the Jini Technology
 * Core Platform Compatibility Kit Version 1.0B
 * 
 * Tester(s): Sun Microsystems, Inc.
 * 
 * Test Platform(s): Java(TM) 2 SDK v1.2.2 Solaris(TM) Reference
 *                   Implementation (Final) on Solaris 2.6
 * 
 *                   Java 2 SDK v1.2.2 Microsoft Windows 95/98/NT
 *                   Production Release (Final) on Microsoft Windows
 *                   NT 4.0 SP4
 * 
 * CopyrightVersion 1.0.1_JXP
 */
package net.jini.discovery;

import net.jini.core.discovery.LookupLocator;
import java.io.InputStream;
import java.io.OutputStream;
import java.io.IOException;
import java.io.InterruptedIOException;
import java.net.DatagramPacket;
import java.net.MulticastSocket;
import java.net.InetAddress;
import java.net.ServerSocket;
import java.net.Socket;
import java.security.AccessControlContext;
import java.security.AccessController;
import java.security.PrivilegedAction;
import java.security.PrivilegedActionException;
import java.security.PrivilegedExceptionAction;
import java.util.Collection;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.Map;
import java.util.Set;
import net.jini.core.lookup.ServiceID;
import net.jini.core.lookup.ServiceRegistrar;

/**
 * This class allows one to find instances of the lookup
 * service that are nearby on the network, using group names.
 * It provides a simple unified front end to the multicast request
 * and announcement protocols.
 *
 * @see net.jini.core.lookup.ServiceRegistrar
 * @see DiscoveryListener
 * @see DiscoveryEvent
 * @see DiscoveryPermission
 * @author Bryan O'Sullivan
 */
public final class LookupDiscovery
{
    /**
     * Do nothing (halt discovery temporarily).
     *
     * @see #LookupDiscovery
     * @see #setGroups
     */
    public static final String[] NO_GROUPS = new String[0];
    /**
     * Attempt to discover all reachable lookup services.
     *
     * @see #LookupDiscovery
     * @see #setGroups
     */
    public static final String[] ALL_GROUPS = null;
    /**
     * Becomes true if the terminate method is called.
     */
    private boolean terminated = false;
    /**
     * The set of listeners for discovery events.
     */
    private ArrayList listeners = new ArrayList(1);
    /**
     * The set of groups in which we are interested.  The empty set
     * denotes "no groups"; null denotes "any group".
     */
    private Set groups = null;
    /**
     * Map from ServiceID to IncomingUnicastResponse.
     */
    private Map registrars = new HashMap(11);
    /**
     * Unicast discoveries that are currently pending.
     */
    private Set pendingDiscoveries = new HashSet(11);
    /**
     * Thread that handles pending notifications.
     */
    private Notifier notifierThread;
    /**
     * Notifications to be sent to listeners.
     */
    private LinkedList pendingNotifies = new LinkedList();
    /**
     * Thread that handles pending unicast discoveries.
     */
    private UnicastDiscoverer discovererThread;
    /**
     * Thread that handles incoming multicast announcements.
     */
    private AnnouncementListener announceeThread;
    /**
     * Collection of Thread objects that represent multicast request
     * handlers.
     */
    private Collection requestors = new LinkedList();
    /**
     * Thread that manages incoming multicast responses.  Created and
     * destroyed on demand.
     */
    private ResponseListener respondeeThread = null;
    /**
     * Access control context to use for listener callbacks.
     */
    private final AccessControlContext acc = AccessController.getContext();

    /**
     * This thread listens for multicast announcements from lookup
     * services.  If it determines that we haven't yet discovered a
     * given service, it marks that service for unicast discovery
     * later.
     */
    private class AnnouncementListener extends Thread {
	/** Multicast socket for receiving packets */
	private MulticastSocket sock;
	
	/** Create a daemon thread */
	public AnnouncementListener() throws IOException {
	    super("multicast discovery announcement listener");
	    setDaemon(true);
	    sock = new MulticastSocket(Constants.discoveryPort);
	    sock.joinGroup(Constants.getAnnouncementAddress());
	}

	/** True if thread has been interrupted */
	private boolean interrupted = false;

	/* This is a workaround for Thread.interrupt not working on
	 * MulticastSocket.receive on all platforms.
	 */
	public void interrupt() {
	    interrupted = true;
	    sock.close();
	}

	public boolean isInterrupted() {
	    return interrupted;
	}

	public void run() {
	    byte[] buf = new byte[512];
	    DatagramPacket pkt = new DatagramPacket(buf, buf.length);

	    while (!isInterrupted()) {
		try {
		    pkt.setLength(buf.length);
		    try {
			sock.receive(pkt);
		    } catch (NullPointerException e) {
			break; // workaround for bug 4190513
		    }
		    IncomingMulticastAnnouncement ann =
			new IncomingMulticastAnnouncement(pkt);
		    // Check to see if we've already discovered this
		    // lookup service or are not interested in it.
		    synchronized (registrars) {
			if (registrars.containsKey(ann.getServiceID()) ||
			    !groupsOverlap(ann.getGroups()))
			    continue;
		    }
		    // Ensure that we're not already trying to
		    // discover this lookup service.
		    synchronized (pendingDiscoveries) {
			if (pendingDiscoveries.add(ann.getLocator()) &&
			    discovererThread == null)
			{
			    discovererThread = new UnicastDiscoverer();
			    discovererThread.start();
			}
		    }
		} catch (InterruptedIOException e) {
		    break;
		} catch (Exception e) {
		    printException(e);
		    // ignore
		}
	    }
	    sock.close();
	    sock = null;
	}
    }

    /**
     * This thread waits for newly-announced lookup services to be
     * added to its queue, and snarfs them via unicast discovery. <p>
     *
     * Why handle unicast discovery asynchronously?  It can take a while
     * for unicast discovery to fail if a lookup service disappears in
     * between making an announcement/connect and discovery starting, and we
     * prefer not to miss multicast announcements or unicast connects.
     */
    private class UnicastDiscoverer extends Thread {
	/** Create a daemon thread */
	public UnicastDiscoverer() {
	    super("unicast discovery");
	    setDaemon(true);
	}

	/** True if thread has been interrupted */
	private boolean interrupted = false;

	/**
	 * We can't normally distinguish between Thread.interrupt() and
	 * timeout from setSoTimeout (see bug 4158021), so we have to
	 * invent our own way to distinguish.
	 */
	public void interrupt() {
	    interrupted = true;
	    super.interrupt();
	}

	public boolean isInterrupted() {
	    return interrupted;
	}

	public void run() {
	    int timeout = 60 * 1000;
	    try {
		timeout = Integer.getInteger("net.jini.discovery.timeout",
					     timeout).intValue();
	    } catch (SecurityException e) {
	    }
	    while (!isInterrupted()) {
		Object req = null;
		try {
		    // If the set of lookups to discover is non-empty,
		    // then find some element from the set, else go away.
		    boolean done = false; // bug 4084783/4187594
		    synchronized (pendingDiscoveries) {
			if (pendingDiscoveries.isEmpty()) {
			    discovererThread = null;
			    done = true; // bug 4084783/4187594
			    return;
			}
			req = pendingDiscoveries.iterator().next();
		    }
		    Socket sock;
		    if (req instanceof Socket)
			sock = (Socket)req;
		    else
			sock = new Socket(((LookupLocator)req).getHost(),
					  ((LookupLocator)req).getPort());
		    IncomingUnicastResponse resp;
		    try {
			sock.setSoTimeout(timeout);
			OutputStream os = sock.getOutputStream();
			InputStream is = sock.getInputStream();
			OutgoingUnicastRequest.marshal(os);
			resp = new IncomingUnicastResponse(is);
		    } finally {
			try {
			    sock.close();
			} catch (IOException e) {
			    // ignore
			}
		    }
		    maybeAddNewRegistrar(resp);
		} catch (InterruptedIOException e) {
		    break;
		} catch (Exception e) {
		    printException(e);
		    // ignore
		} finally {
		    if (req != null) {
			// Remove the lookup service from the set
			// of those awaiting discovery, regardless
			// of whether or not we were successful in
			// discovering it.
			synchronized (pendingDiscoveries) {
			    pendingDiscoveries.remove(req);
			}
		    }
		}
	    }
	    synchronized (pendingDiscoveries) {
		discovererThread = null;
		for (Iterator iter = pendingDiscoveries.iterator();
		     iter.hasNext(); )
		{
		    Object req = iter.next();
		    iter.remove();
		    if (req instanceof Socket) {
			try {
			    ((Socket)req).close();
			} catch (IOException e) {
			    // ignore
			}
		    }
		}
	    }
	}
    }

    /**
     * This thread listens for incoming responses to multicast
     * requests, and then queues them for UnicastDiscoverer to process.
     */
    private class ResponseListener extends Thread {
	/** Server socket for accepting connections */
	public ServerSocket serv;
	
	/** Create a daemon thread */
	public ResponseListener() throws IOException {
	    super("multicast discovery response listener");
	    setDaemon(true);
	    serv = new ServerSocket(0);
	}

	/** True if thread has been interrupted */
	private boolean interrupted = false;

	/* This is a workaround for Thread.interrupt not working on
	 * ServerSocket.accept on all platforms.  ServerSocket.close
	 * can't be used as a workaround, because it also doesn't work
	 * on all platforms.
	 */
	public void interrupt() {
	    interrupted = true;
	    try {
		(new Socket(InetAddress.getLocalHost(), getPort())).close();
	    } catch (IOException e) {
	    }
	}

	public boolean isInterrupted() {
	    return interrupted;
	}

	public void run() {
	    while (!isInterrupted()) {
		try {
		    Socket sock = serv.accept();
		    if (isInterrupted()) {
			try {
			    sock.close();
			} catch (IOException e) {
			}
			break;
		    }
		    synchronized (pendingDiscoveries) {
			pendingDiscoveries.add(sock);
			if (discovererThread == null) {
			    discovererThread = new UnicastDiscoverer();
			    discovererThread.start();
			}
		    }
		} catch (InterruptedIOException e) {
		    break;
		} catch (Exception e) {
		    printException(e);
		    // ignore with impunity
		}
	    }
	    try {
		serv.close();
	    } catch (IOException e) {
		printException(e);
		// ignore
	    }
	}

	/** Return the local port of the socket */
	public int getPort() {
	    return serv.getLocalPort();
	}
    }

    /**
     * This thread sends out multicast requests periodically for
     * a limited period of time.
     */
    private class Requestor extends Thread {
	/** Multicast socket for sending packets */
	private MulticastSocket sock;
	/** Unicast response port */
	private int responsePort;
	/** Groups to request */
	private String[] groups;

	/** Create a daemon thread */
	public Requestor(String[] groups, int port) throws IOException {
	    super("multicast discovery request");
	    setDaemon(true);
	    sock = new MulticastSocket(Constants.discoveryPort);
	    int ttl = 15;
	    try {
		ttl = Integer.getInteger("net.jini.discovery.ttl",
					 ttl).intValue();
	    } catch (SecurityException e) {
	    }
	    sock.setTimeToLive(ttl);
	    responsePort = port;
	    this.groups = groups == null ? new String[0] : groups;
	}

	/**
	 * It would be better to not sleep at the end, and have the
	 * ResponseListener use setSoTimeout on the server socket and
	 * go away by itself when its been idle for a while with no
	 * requestors, but using setSoTimeout in that way can cause
	 * random hangs on the Solaris(TM) operating system.
	 */
	public void run() {
	    int count; // bug 4084783/4187594
	    try {
		for (count = 7; --count >= 0 && !isInterrupted(); ) {
		    try {
			DatagramPacket[] reqs = 
			    OutgoingMulticastRequest.marshal(responsePort,
							     groups,
							     getServiceIDs());
			for (int i = 0; i < reqs.length; i++) {
			    try {
				sock.send(reqs[i]);
			    } catch (InterruptedIOException e) {
				return;
			    } catch (Exception e) {
				printException(e);
				// relatively safe to ignore
			    }
			}
		    } catch (InterruptedIOException e) {
			break;
		    } catch (Exception e) {
			printException(e);
			// @@@ what to do here?
		    }
		    Thread.sleep(count > 0 ? 5000 : 2 * 60 * 1000);
		}
	    } catch (InterruptedException e) {
		printException(e);
		// die gracefully
	    } finally {
		synchronized (requestors) {
		    requestors.remove(Thread.currentThread());
		    if (respondeeThread != null && requestors.isEmpty()) {
			respondeeThread.interrupt();
			respondeeThread = null;
		    }
		}
		sock.close();
	    }
	}
    }

    /** Simple holder for notify task data */
    private static class NotifyTask {
	/** The listeners to notify */
	public final ArrayList listeners;
	/** The proxies that have been discovered or discarded */
	public final ServiceRegistrar[] regs;
	/** True if discarded, else discovered */
	public final boolean discard;

	public NotifyTask(ArrayList listeners,
			  ServiceRegistrar[] regs,
			  boolean discard)
	{
	    this.listeners = listeners;
	    this.regs = regs;
	    this.discard = discard;
	}
    }

    /** Task thread for calling listeners with notifications */
    private class Notifier extends Thread {

	/** Create a daemon thread */
	public Notifier() {
	    super("event listener notification");
	    setDaemon(true);
	}

	public void run() {
	    while (true) {
		final NotifyTask task;
		synchronized (pendingNotifies) {
		    if (pendingNotifies.isEmpty()) {
			notifierThread = null;
			return;
		    }
		    task = (NotifyTask)pendingNotifies.removeFirst();
		}
		AccessController.doPrivileged(new PrivilegedAction() {
		    public Object run() {
			for (Iterator iter = task.listeners.iterator();
			     iter.hasNext(); )
			{
			    DiscoveryEvent e =
				new DiscoveryEvent(LookupDiscovery.this,
						   (ServiceRegistrar[])
						   task.regs.clone());
			    DiscoveryListener l =
				(DiscoveryListener)iter.next();
			    if (task.discard)
				l.discarded(e);
			    else
				l.discovered(e);
			}
			return null;
		    }
		}, acc);
	    }
	}
    }

    /** Print the exception stack trace if debugging is enabled. */
    private static void printException(Exception e) {
	try {
	    if (System.getProperty("net.jini.discovery.debug") != null) {
		e.printStackTrace();
	    }
	} catch (SecurityException ee) {
	}
    }

    /**
     * If the given lookup service is not already in the set we have
     * discovered, we add it here, and notify listeners as
     * appropriate.
     *
     * @param resp the response to add
     */
    private void maybeAddNewRegistrar(IncomingUnicastResponse resp) {
	synchronized (registrars) {
	    if (groupsOverlap(resp.groups) &&
		!resp.equals(registrars.put(resp.registrar.getServiceID(),
					    resp)) &&
		!listeners.isEmpty())
	    {
		addNotify((ArrayList)listeners.clone(),
			  new ServiceRegistrar[]{resp.registrar}, false);
	    }
	}
    }

    /**
     * Add a notification task to the pending queue, and start a
     * notifierThread if one isn't running.
     */
    private void addNotify(ArrayList notifies,
			   ServiceRegistrar[] regs,
			   boolean discard)
    {
	synchronized (pendingNotifies) {
	    pendingNotifies.addLast(new NotifyTask(notifies, regs, discard));
	    if (notifierThread == null) {
		AccessController.doPrivileged(new PrivilegedAction() {
		    public Object run() {
			notifierThread = new Notifier();
			notifierThread.start();
			return null;
		    }
		});
	    }
	}
    }

    /**
     * Indicate whether any of the groups named in possibilities
     * matches any of the groups we're interested in discovering.
     *
     * @param possibilities - the set of groups we may be interested
     * (must not be null)
     */
    private boolean groupsOverlap(String[] possibilities) {
	// Match if we're interested in any group, or if we're
	// interested in none and there are no possibilities.
	if (groups == null)
	    return true;
	for (int i = 0; i < possibilities.length; i++) {
	    if (groups.contains(possibilities[i]))
		return true;
	}
	return false;
    }

    /**
     * Construct a new lookup discovery object, set to discover the
     * given set of groups.  The set is represented as an array of
     * strings.  This array may be empty, which is taken as the empty
     * set, and discovery is not performed.  The reference passed in
     * may be null, which is taken as no set, and in which case
     * discovery of all reachable lookup services is performed.
     * Otherwise, the array contains the names of groups to discover.
     * The caller must have DiscoveryPermission for each group (or
     * for all groups, if the array is null).
     *
     * @param groups the set of group names to discover (null for no
     * set, empty for no discovery)
     * @see #NO_GROUPS
     * @see #ALL_GROUPS
     * @see #setGroups
     * @see DiscoveryPermission
     * @exception IOException an error occurred in starting discovery
     */
    public LookupDiscovery(String[] groups) throws IOException {
	checkGroups(groups);
	if (groups != null) {
	    this.groups = new HashSet(groups.length * 2);
	    for (int i = 0; i < groups.length; i++) {
		this.groups.add(groups[i]);
	    }
	}
	try {
	    AccessController.doPrivileged(new PrivilegedExceptionAction() {
		public Object run() throws Exception {
		    announceeThread = new AnnouncementListener();
		    return null;
		}
	    });
	} catch (PrivilegedActionException e) {
	    throw (IOException)e.getException();
	}
	if (this.groups == null || !this.groups.isEmpty())
	    requestGroups(this.groups);
	announceeThread.start();
    }

    /**
     * Check that the caller has discovery permission for each group.
     */
    private static void checkGroups(String[] groups) {
	SecurityManager sm = System.getSecurityManager();
	if (sm == null)
	    return;
	if (groups != null) {
	    for (int i = 0; i < groups.length; i++) {
		sm.checkPermission(new DiscoveryPermission(groups[i]));
	    }
	} else {
	    sm.checkPermission(new DiscoveryPermission("*"));
	}
    }

    /**
     * Interrupt all currently-running threads.
     */
    private void nukeThreads() {
	AccessController.doPrivileged(new PrivilegedAction() {
	    public Object run() {
		synchronized (requestors) {
		    for (Iterator iter = requestors.iterator();
			 iter.hasNext(); )
		    {
			Thread t = (Thread) iter.next();
			t.interrupt();
		    }
		    if (respondeeThread != null)
			respondeeThread.interrupt();
		}
		announceeThread.interrupt();
		synchronized (pendingDiscoveries) {
		    if (discovererThread != null)
			discovererThread.interrupt();
		}
		return null;
	    }
	});
    }
    
    /**
     * Register a listener as interested in receiving DiscoveryEvent
     * notifications.
     *
     * @param l the listener to register
     * @see DiscoveryEvent
     */
    public void addDiscoveryListener(DiscoveryListener l) {
	synchronized (registrars) {
	    if (listeners.indexOf(l) >= 0)
		return;
	    listeners.add(l);
	    if (registrars.isEmpty())
		return;
	    Iterator iter = registrars.values().iterator();
	    ServiceRegistrar[] regs = new ServiceRegistrar[registrars.size()];
	    for (int i = 0; iter.hasNext(); i++) {
		regs[i] = ((IncomingUnicastResponse)iter.next()).registrar;
	    }
	    ArrayList list = new ArrayList(1);
	    list.add(l);
	    addNotify(list, regs, false);
	}
    }

    /**
     * Indicate that a listener is no longer interested in receiving
     * DiscoveryEvent notifications.
     *
     * @param l the listener to unregister
     */
    public void removeDiscoveryListener(DiscoveryListener l) {
	synchronized (registrars) {
	    listeners.remove(l);
	}
    }

    /**
     * Discard a registrar from the set of registrars already
     * discovered.  This does not prevent that registrar from being
     * rediscovered; it is intended to be used to clear unreachable
     * entries from the set. <p>
     *
     * If the registrar has been discovered using this LookupDiscovery
     * object, each listener registered with this object will have its
     * discarded method called with the given registrar as parameter.
     *
     * @param reg the registrar to discard
     * @see DiscoveryListener#discarded
     */
    public void discard(ServiceRegistrar reg) {
	synchronized (registrars) {
	    if (registrars.remove(reg.getServiceID()) != null &&
		!listeners.isEmpty())
	    {
		addNotify((ArrayList)listeners.clone(),
			  new ServiceRegistrar[]{reg}, true);
	    }
	}
    }

    /**
     * Return the set of group names this LookupDiscovery instance is
     * trying to discover.  If this method returns the empty array,
     * that value is guaranteed to be referentially equal to
     * LookupDiscovery.NO_GROUPS.
     *
     * @return the set of groups to be discovered (null for all, empty
     * for no discovery)
     * @see #NO_GROUPS
     * @see #ALL_GROUPS
     */
    public String[] getGroups() {
	synchronized (registrars) {
	    if (groups == null)
		return ALL_GROUPS;
	    if (groups.isEmpty())
		return NO_GROUPS;
	    return collectionToStrings(groups);
	}
    }

    /**
     * Return the set of service IDs we have discovered to date.
     */
    private ServiceID[] getServiceIDs() {
	synchronized (registrars) {
	    return (ServiceID[])
		registrars.keySet().toArray(new ServiceID[registrars.size()]);
	}
    }

    /**
     * Change the set of groups to be discovered to correspond to the
     * given set.  The set is represented as an array of strings.
     * This array may be empty, which is taken as the empty set, and
     * discovery is not performed.  The reference passed in may be
     * null, which is taken as no set, and in which case discovery of
     * all reachable lookup services is performed.  Otherwise, the
     * array contains the names of groups to discover.
     * The caller must have DiscoveryPermission for each group (or
     * for all groups, if the array is null).
     *
     * @param newGroups the new set of groups to discover (null for
     * all, empty array for no discovery)
     * @exception IOException an error occurred when starting
     * multicast discovery
     * @exception IllegalStateException this method was called after
     * the terminate method was called
     * @see #LookupDiscovery
     * @see #ALL_GROUPS
     * @see #NO_GROUPS
     * @see DiscoveryPermission
     */
    public void setGroups(String[] newGroups) throws IOException {
	checkGroups(newGroups);
	boolean maybeDiscard = false;
	Set newGrps = null;
	if (newGroups != null) {	
	    newGrps = new HashSet(newGroups.length * 2);
	    for (int i = 0; i < newGroups.length; i++) {
		newGrps.add(newGroups[i]);
	    }
	}
	synchronized (registrars) {
	    if (terminated)
		throw new IllegalStateException("discovery terminated");
	    if (newGroups == null) {
		if (groups != null) {
		    groups = null;
		    requestGroups(null);
		}
		return;
	    }
	    if (groups == null) {
		groups = new HashSet(11);
		maybeDiscard = true;
	    }
	    Set toAdd = new HashSet(newGrps);
	    toAdd.removeAll(groups);
	    // Figure out which groups to get rid of.  We start off
	    // with the full set for which we are already listening,
	    // and eliminate any that are in both the new set and the
	    // current set.
	    Collection toRemove = new HashSet(groups);
	    toRemove.removeAll(newGrps);
	    // Add new groups before we remove any old groups, because
	    // removeGroups will start a new round of multicast requests
	    // if the set of groups becomes empty, and we don't want it
	    // to do so without reason.
	    groups.addAll(toAdd);
	    if (!toRemove.isEmpty())
		maybeDiscard |= removeGroupsInt(collectionToStrings(toRemove));
	    if (!toAdd.isEmpty())
		requestGroups(toAdd);
	}
	if (maybeDiscard)
	    maybeDiscardRegistrars();
    }

    /**
     * Check the registrars we've previously discovered, and discard
     * all that are no longer members of any of our groups.
     */
    private void maybeDiscardRegistrars() {
	synchronized (registrars) {
	    Collection byebye = new ArrayList(1);
	    for (Iterator iter = registrars.values().iterator();
		 iter.hasNext(); )
	    {
		IncomingUnicastResponse ent =
		    (IncomingUnicastResponse) iter.next();
		if (!groupsOverlap(ent.groups)) {
		    byebye.add(ent.registrar);
		    iter.remove();
		}
	    }
	    if (!byebye.isEmpty() && !listeners.isEmpty()) {
		ServiceRegistrar[] nuked = new ServiceRegistrar[byebye.size()];
		byebye.toArray(nuked);
		addNotify((ArrayList)listeners.clone(), nuked, true);
	    }
	}
    }

    /**
     * Start multicast requesting a given set of groups.
     */
    private void requestGroups(final Collection req) throws IOException {
	try {
	    AccessController.doPrivileged(new PrivilegedExceptionAction() {
		public Object run() throws Exception {
		    Thread t;
		    synchronized (requestors) {
			if (respondeeThread == null) {
			    respondeeThread = new ResponseListener();
			    respondeeThread.start();
			}
			t = new Requestor(collectionToStrings(req),
					  respondeeThread.getPort());
			requestors.add(t);
		    }
		    t.start();
		    return null;
		}
	    });
	} catch (PrivilegedActionException e) {
	    throw (IOException)e.getException();
	}
    }

    /**
     * Add a set of groups to the set to be discovered.
     * The caller must have DiscoveryPermission for each group.
     *
     * @param newGroups the groups to add
     * @exception IOException the multicast request protocol failed to start
     * @exception IllegalStateException this method was called after
     * the terminate method was called
     * @exception UnsupportedOperationException there is no set of
     * groups to add to
     * @see DiscoveryPermission
     */
    public void addGroups(String[] newGroups) throws IOException {
	checkGroups(newGroups);
	synchronized (registrars) {
	    if (terminated)
		throw new IllegalStateException("discovery terminated");
	    if (groups == null)
		throw new UnsupportedOperationException(
					  "can't add to \"any groups\"");
	    Collection req = new ArrayList(newGroups.length);
	    for (int i = 0; i < newGroups.length; i++) {
		if (groups.add(newGroups[i]))
		    req.add(newGroups[i]);
	    }
	    if (!req.isEmpty())
		requestGroups(req);
	}
    }

    /**
     * Remove a set of groups from the set to be discovered.
     *
     * @param oldGroups groups to remove
     * @exception IllegalStateException this method was called after
     * the terminate method was called
     * @exception UnsupportedOperationException there is no set of
     * groups from which to remove
     */
    public void removeGroups(String[] oldGroups) {
	boolean maybeDiscard;
	synchronized (registrars) {
	    if (terminated)
		throw new IllegalStateException("discovery terminated");
	    if (groups == null)
		throw new UnsupportedOperationException(
					   "can't remove from \"any groups\"");
	    maybeDiscard = removeGroupsInt(oldGroups);
	}
	if (maybeDiscard)
	    maybeDiscardRegistrars();
    }

    /**
     * Remove the specified groups, and return true if any were
     * actually removed.
     */
    private boolean removeGroupsInt(String[] oldGroups) {
	boolean removed = false;
	for (int i = 0; i < oldGroups.length; i++) {
	    removed |= groups.remove(oldGroups[i]);
	}
	return removed;
    }

    /**
     * Terminate the discovery process.
     */
    public void terminate() {
	synchronized (registrars) {
	    if (terminated)
		return;
	    terminated = true;
	}
	nukeThreads();
    }

    public void finalize() {
	terminate();
    }

    /** Convert a collection to an array of strings. */
    private static final String[] collectionToStrings(Collection c) {
	return c == null ? null : (String[]) c.toArray(new String[c.size()]);
    }
}
